import logging

from src.mmutil import Util


# -----------------------------------------------------------------------------
#
#   Name:      MMLookup
#
#   Purpose:   Does key/value pair lookup functions.
#
#
#   Usage:     mylookup = MMLookup(mycsv, data_path=path_to_csv)
#              myval = mylookup.key_to_val(mykey)
#
#
#   Caution:   This object assumes the key is always the 0th field
#
#   Unit Test: cities = mm.MMLookup('Cities.csv', data_path='../data/stage1/')
#              print(cities.get_col_count())
#              print(cities.get_col_names())
#              print(cities.key_to_val('City', 4001))
#              print(cities.val_to_key('State', 'TX'))
#
# -----------------------------------------------------------------------------
class MMLookup:

    def __init__(self, source='', path='', local_root='./', gcs_root='ops/'):
        self.df = Util.get_source_data(source=source, path=path, local_root=local_root, gcs_root=gcs_root)
        self.col_names = self.df.columns
        self.col_count = len(self.df.columns)

    def get_col_count(self):
        return self.col_count

    def get_col_names(self):
        return self.col_names

    # should always return a unique key
    def key_to_val(self, column, key):
        try:
            val = self.df.loc[self.df[self.df.columns[0]] == key]  # assumes the key is always the 0th field
            return val[column].values[0]
        except IndexError:
            logging.warning(f"Key: {key} not found!, returning -1")
            return -1

    # returns a list of keys to rows with the supplied value in the given column
    def val_to_key(self, column, value):
        try:
            key = self.df.loc[self.df[column] == value][self.df.columns[0]]  # assumes the key is always the 0th field
        except IndexError:
            raise ValueError(f"value {value} not found")

        return key.values

    def replace_ids(self, df, from_id, to_col_name):
        df['T1_' + to_col_name] = df.apply(lambda x: self.key_to_val(to_col_name, x['T1_' + from_id]), axis=1)
        df['T2_' + to_col_name] = df.apply(lambda x: self.key_to_val(to_col_name, x['T2_' + from_id]), axis=1)
        return df
